function [ TE, sigmasOut ] = estimTransferEn( y_tu, y_t, x_t, sigmas, restart )
% function [ TE, sigmasOut ] = estimTransferEn( y_tu, y_t, x_t, sigmas )
%    Estimates transfer entropy from a time series x_t to a time series
%    y_t, trying to predict y_{t+u} according to [Vicente_JComputNeurosci2011].
%    All the histories must be in column vectors.

	if nargin < 4
		sigmas = [];
	end
	if nargin < 5
		restart = false;
	end

	% the transfer entropy is given as
	% TE(X→Y)=S(y_t,x_t) − S(yt+u,yt,xt) + S(yt+u,yt) − S(yt)
	% where Ss are Shannon entropies (we use differential entropies
	% instead)
	
	% S(yt+u,yt,xt)
	if isempty(sigmas)
		s = [];
	else
		s = sigmas{1};
	end
	[Syyx, s] = estimEntropy([flipud(y_tu); flipud(y_t); flipud(x_t)], s, restart);
	sigmasOut{1} = s;
	
	% S(y_t,x_t)
	if isempty(sigmas)
		s = [];
	else
		s = sigmas{2};
	end
	[Syx, s] = estimEntropy([y_t; x_t], s, restart);
	sigmasOut{2} = s;
	
	% S(yt+u,yt)
	if isempty(sigmas)
		s = [];
	else
		s = sigmas{3};
	end
	[Syy, s] = estimEntropy([y_tu; y_t], s, restart);
	sigmasOut{3} = s;
	
	% S(yt)
	if isempty(sigmas)
		s = [];
	else
		s = sigmas{4};
	end
	[Sy, s] = estimEntropy(y_t, s, restart);
	sigmasOut{4} = s;
	
	TE = Syx - Syyx + Syy - Sy;
	
% 	fprintf('Syyx(%d) = %.1f, Syx(%d) = %.1f, Syy(%d) = %.1f, Sy(%d) = %.1f\n',...
% 		size([y_tu; y_t; x_t],1), Syyx, size([y_t; x_t],1), Syx, size([y_tu; y_t],1), Syy, size(y_t,1), Sy);
	
end

