function [ sigma, converged, t ] = sigmaEM( S, sigmaInit, changeThr, maxIters)
%SIGMAEM Summary of this function goes here
%    EM algorithm to estimate an optimal sigma (cov. matrix) for KDE for
%    entropy estimation based on [Hlavackova-Schindler_PhysicsReports_2007]

	d = size(S, 1);
	if nargin < 2 || isempty(sigmaInit)
		sigmaInit = cov(S');
	end
	if nargin < 3
		changeThr = 1e-6;
	end
	if nargin < 4 || maxIters == 0
		maxIters = 100;
	end
	
	if size(S, 2) <= 1
		sigma = eye(d);
		return
	end
	
	converged = false;
	
	sigma = sigmaInit;
	logLike = -inf;
	for t = 1:maxIters % let's set an upper bound just to be sure
		sigmaOld = sigma;
		logLikeOld = logLike;
		
% 		[sigma, logLike] = updateSigma(S, sigmaOld);
		[sigma, logLike] = fUpdateCovarianceKDE(S, sigmaOld); % mex-file not available in the published code, use the above
		
		changeNow = (logLike - logLikeOld) / abs(logLike);

		% //TODO do this only when in lo-dim space!
		% (unless we have a shitload of data!)
% 		sigma = sigma / sigmaOld * sigma; % overrelaxation

% 		fprintf('Iteration #%i, logLike %15.7f, relative change %g...\n', t, logLike, changeNow);
		if changeNow < changeThr
			converged = true;
			break
		end
		if any(isnan(sigma(:))) || rcond(sigma) < eps
			sigma = sigmaInit;
			break;
		end
	end
	
end



