function [p] = ttestWelch(signalA, signalB, oneSided)
% function [p] = ttestWelch(signalA, signalB, oneSided)
%    Tests if two normal distributions with unknow variances have
%    the same mean (according to https://en.wikipedia.org/wiki/Welch%27s_t_test).
	
	if nargin < 3
		oneSided = true;
	end
	
	signalA = signalA(:);
	signalA(isnan(signalA) | isinf(signalA) | imag(signalA) > eps) = [];
	signalA = real(signalA);
% 	s = std(signalA);
% 	m = mean(signalA);
% 	signalA(signalA > m+3*s | signalA < m-3*s) = [];
	
	signalB = signalB(:);
	signalB(isnan(signalB) | isinf(signalB) | imag(signalB) > eps) = [];
	signalB = real(signalB);
% 	s = std(signalB);
% 	m = mean(signalB);
% 	signalB(signalB > m+3*s | signalB < m-3*s) = [];
	
	nA = length(signalA);
	nB = length(signalB);
	
	
	vA = var(signalA);
	vB = var(signalB);
	
	T = ( mean(signalA) - mean(signalB) ) / ( sqrt( vA/nA + vB/nB ) );
	DoF = floor((vA/nA + vB/nB)^2 / (vA^2/nA^2/(nA-1) + vB^2/nB^2/(nB-1)));
	
	if oneSided
		p = tcdf(-T, DoF);
	else
		p = 2 * tcdf(-abs(T), DoF);
	end
	
end


